# encoding: utf-8

from distutils.core import setup

setup(name='audiocalc',
      version='0.0.4',
      description='A few audio/sound calculation utilities',
      author='Marian Steinbach',
      author_email='marian@sendung.de',
      url='https://github.com/marians/audiocalc',
      packages=['audiocalc'],
      requires=[])
